/***************************************************************************
                          musicfilenamelayout.h  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUSICFILENAMELAYOUT_H
#define MUSICFILENAMELAYOUT_H

class QString;
class QRegExp;
class MusicInfo;

#include <qvaluelist.h>
#include <qregexp.h>

#include "normalizationparameters.h"


/**This class is a base class for MP3 (or Ogg) filename layout. Derived classes will define RegExps and methods to match etc filenames like:
Artist Name - Song Name.mp3
03 - Song Name.mp3
etc
  *@author Avi Alkalay
  */

class MusicFilenameLayout {
	public:
	public: 
		MusicFilenameLayout(QString _name, QString _tpl, QRegExp _exp);
		MusicFilenameLayout(QString _name, QString _tpl, QString _exp=0L, bool minimal=false);
		MusicFilenameLayout();
		~MusicFilenameLayout();
		/** Tests if the filename matches this layout

		@param filename the file name to be tested
		@returns true if filename matches */
		bool match(const QString &filename);

		

		/** Tries to find some music information from file name

		@param filename the file name to be tested
		@returns the MusicInfo object containing as much as was matched
		*/
		MusicInfo getInfo(const QString &filename);


		/** Returns the passed string substituting _ with space or vice-versa.
		Default is to remove underscore.*/
		static QString toggleUnderline(const QString &filename,bool underscore=false);

		/** Returns the passed string substituting _ with space. */
		static QString noUnderline(const QString &filename);

		/** Returns the passed string substituting space with _. */
		static QString yesUnderline(const QString &filename);

		/** Normalize a filename to a standard:
		- Space to underline
		- Caps first letter of each word
		- Small caps extension
		@returns True if something changed.*/
		static bool normalize(QString &filename,NormalizationParameters &norm);

		static QString getExtension(const QString &filename);

		/** @returns The filename constructed from the template (%a - %n.%e) and the MusicInfo object. */
		QString getNewFilename(MusicInfo info, NormalizationParameters &np);



		
	public: // Public attributes
		/** This filename layout name. Something like "Artist Name - Song Name.mp3". */
		QString name;

	private: // Private attributes
		/** The template to reproduce a filename layout. Ex: "%a - %n.%e".
		The macros are like this:

		%a == Artist
		%A == Album
		%n == Song
		%t == Track
		%b == Bitrate
		%e == Extension
		%y == Year
	*/
		QString tpl;


		/** Position of each item in the matched string*/
		int trackPos,titlePos,artistPos,albumPos,yearPos,extensionPos,bitratePos;


		/** The regular expression that will match a file name. */
		QRegExp regexp;

		
		/** The global regular expression that holds all extensions. */
		static QString extensions;

	private:
		/** Add the extension to the regexp */
		void addExtensions();

		/** Guesses the titlePos, albumPos, artistPos, trackPos and extPos from the template. */
		void setSubExpPos(void);
};








class FilenameMiner : public QValueList<MusicFilenameLayout> {
	public:
		FilenameMiner();
		~FilenameMiner();
		bool match(const QString &filename);
		MusicInfo mine(const QString &filename);
		/** Returns the list of layout representations. */
		QStringList getLayouts();
	private: // Private methods
		/** Layouts initialization workhorse. */
		void initLayouts();
};

#endif
